import enum
import lambdaskill.skill as skill


class PLAY_BEHAVIOR(enum.Enum):
    ENQUEUE = "ENQUEUE"
    REPLACE_ALL = "REPLACE_ALL"
    REPLACE_ENQUEUED = "REPLACE_ENQUEUED"


class CLEAR_BEHAVIOR(enum.Enum):
    CLEAR_ENQUEUED = "CLEAR_ENQUEUED"
    CLEAR_ALL = "CLEAR_ALL"


class StopDirective(skill.Directive):

    directive_type = 'AudioPlayer.Stop'


class ClearQueueDirective(skill.Directive):

    directive_type = 'AudioPlayer.ClearQueue'

    def __init__(self, clear_behavior):
        super(ClearQueueDirective, self).__init__()
        self.clear_behavior = CLEAR_BEHAVIOR(clear_behavior)

    def prepare(self):
        return {
            'type': type(self).directive_type,
            'clearBehavior': self.clear_behavior.value
        }


class PlayDirective(skill.Directive):

    directive_type = 'AudioPlayer.Play'

    def __init__(self, audio_item, play_behavior):
        super(PlayDirective, self).__init__()
        self.audio_item = audio_item
        self.play_behavior = PLAY_BEHAVIOR(play_behavior)

    def prepare(self):
        return {
            'type': type(self).directive_type,
            'audioItem': self.audio_item.prepare(),
            'playBehavior': self.play_behavior.value
        }


class AudioMetaData(object):

    def __init__(self, title=None, subtitle=None, art=None, background=None):
        self.title = title
        self.subtitle = subtitle
        self.art = art
        self.background = background

    def prepare(self):
        metadata = {
            'title': self.title,
            'subtitle': self.subtitle
        }
        metadata = {k: v for k, v in metadata.items() if v}
        if self.art:
            metadata['art'] = self.art.prepare()
        if self.background:
            metadata['backgroundImage'] = self.background.prepare()


class AudioItem(object):

    def __init__(self, url, token, expected_previous_token=None, offset=0, metadata=None):
        self.url = url
        self.token = token
        self.expected_previous_token = expected_previous_token
        self.offset = offset
        self.metadata = metadata

    def add_metadata(self, title=None, subtitle=None, art=None, background=None):
        self.metadata = AudioMetaData(title=title, subtitle=subtitle,
                                      art=art, background=background)
        return self

    def with_metadata(self, metadata):
        self.metadata = metadata
        return self

    def prepare(self):
        audioitem = {
            'stream': {
                'url': self.url,
                'token': self.token,
                'expectedPreviousToken': self.expected_previous_token,
                'offset': self.offset
            }
        }

        if self.metadata:
            audioitem['metadata'] = self.metadata.prepare()

        return audioitem


class AudioSkill(skill.Skill):

    def __init__(self):
        super(AudioSkill, self).__init__()

    def on_audioplayer_playback_started(self, request):
        return

    def on_audioplayer_playback_finished(self, request):
        return

    def on_audioplayer_playback_stopped(self, request):
        return

    def on_audioplayer_playback_nearly_finished(self, request):
        return

    def on_audioplayer_playback_failed(self, request):
        return

    def do_AMZ_pauseintent(self, request):
        raise NotImplementedError()
    
    def do_AMZ_resumeintent(self, request):
        raise NotImplementedError()
    
    def do_AMZ_cancelintent(self, request):
        return NotImplementedError()
    
    def do_AMZ_nextintent(self, request):
        return skill.Response.finish('Track skipping is not implemented for this skill.')
    
    def do_AMZ_previousintent(self, request):
        return skill.Response.finish('Track skipping is not implemented for this skill.')

    def do_AMZ_startoverintent(self, request):
        return skill.Response.finish('Restarting a track is not implemented for this skill.')

    def do_AMZ_shuffleoffintent(self, request):
        return skill.Response.finish('Shuffling is not implemented for this skill.')
    
    def do_AMZ_shuffleonintent(self, request):
        return skill.Response.finish('Shuffling is not implemented for this skill.')
    
    def do_AMZ_repeatintent(self, request):
        return skill.Response.finish('Repeating is not implemented for this skill.')
    
    def do_AMZ_loopoffintent(self, request):
        return skill.Response.finish('Looping is not implemented for this skill.')

    def do_AMZ_looponintent(self, request):
        return skill.Response.finish('Looping is not implemented for this skill.')

    def dispatch(self, request):
        request_handlers = {
            'AudioPlayer.PlaybackStarted': self.on_audioplayer_playback_started,
            'AudioPlayer.PlaybackFinished': self.on_audioplayer_playback_finished,
            'AudioPlayer.PlaybackStopped': self.on_audioplayer_playback_stopped,
            'AudioPlayer.PlaybackNearlyFinished': self.on_audioplayer_playback_nearly_finished,
            'AudioPlayer.PlaybackFailed': self.on_audioplayer_playback_failed
        }
        handler = request_handlers.get(request.request_type, None)
        if handler:
            return handler(request=request)
        return super(AudioSkill, self).dispatch(request=request)



import aniso8601
import datetime
import re


SEASON_DICT = {'WI': '12', 'SP': '03', 'SU': '06', 'FA': '09'}
SEASON_PATTERN = re.compile('^\d{4}-(WI|SP|SU|FA)$')
WEEKEND_PATTERN = re.compile('^\d{4}-W\d{2}-WE$')
TIME_DICT = {'AM': ((0, 0, 0), (11, 59, 59)),
             'PM': ((12, 0, 0), (23, 59, 59)),
             'MO': ((0, 0, 0), (11, 59, 59)),
             'AF': ((12, 0, 0), (16, 59, 59)),
             'EV': ((17, 0, 0), (19, 59, 59)),
             'NI': ((20, 0, 0), (23, 59, 59))}


def parse_date(amzdate):
    if SEASON_PATTERN.search(amzdate):
        start = aniso8601.parse_date('{}-{}-01'.format(amzdate[:4], SEASON_DICT[amzdate[-2:]]))
        yd = (start.month + 3) // 12
        md = (start.month + 3) % 12
        end = datetime.datetime(year=start.year + yd,
                                month=start.month + md,
                                day=1) - datetime.timedelta(days=1)
        return start, end
    elif WEEKEND_PATTERN.search(amzdate):
        return aniso8601.parse_date(amzdate[:-3] + "-6"), aniso8601.parse_date(amzdate[:-3] + "-7")
    elif amzdate.endswith('XX'):
        start = aniso8601.parse_date('{}{}'.format(amzdate[:2], '00'))
        end = datetime.datetime(year=start.year + 100,
                                month=1,
                                day=1) - datetime.timedelta(days=1)
        return start, end
    elif amzdate.endswith('X'):
        start = aniso8601.parse_date('{}{}'.format(amzdate[:3], '0'))
        end = datetime.datetime(year=start.year + 10,
                                month=1,
                                day=1) - datetime.timedelta(days=1)
        return start, end
    elif amzdate == 'PRESENT_REF':
        start = datetime.datetime.today().replace(hour=0, minute=0, second=0, microsecond=0)
        end = start + datetime.timedelta(days=1)
        return start, end
    return aniso8601.parse_date(amzdate)


def parse_time(amztime):
    if amztime in TIME_DICT:
        return tuple(datetime.time(*tt) for tt in TIME_DICT[amztime])
    return aniso8601.parse_time(amztime)


def parse_duration(amzduration):
    return aniso8601.parse_duration(amzduration)


def sequence_to_oxford_string(l):
    if len(l) > 2:
        return ', '.join([str(i) for i in l[:-1]]) + ", and " + str(l[-1])
    elif len(l) == 2:
        return ' and '.join([str(i) for i in l])
    elif len(l) > 0:
        return str(l[0])



import enum
import logging


class PLAYER_ACTIVITY(enum.Enum):
    IDLE = "IDLE"
    PAUSED = "PAUSED"
    PLAYING = "PLAYING"
    BUFFER_UNDERRUN = "BUFFER_UNDERRUN"
    FINISHED = "FINISHED"
    STOPPED = "STOPPED"
    
    
logger = logging.getLogger('lambdaskill')
logger.addHandler(logging.StreamHandler())
if logger.level == logging.NOTSET:
    logger.setLevel(logging.WARNING)


class SSML(object):

    def __init__(self, content):
        self.content = content


class Directive(object):

    directive_type = None

    def prepare(self):
        return {'type': self.directive_type}


class Card(object):

    def __init__(self, title, content=None, text=None, small_image_url=None, large_image_url=None):
        self.title = title
        self.content = content
        self.text = text
        self.small_image_url = small_image_url
        self.large_image_url = large_image_url

    def prepare(self):
        if self.text is not None:
            card = {
                'type': 'Standard',
                'title': self.title,
                'text': self.text
            }
            if any([self.small_image_url, self.large_image_url]):
                image = {k: v for k, v in zip(['smallImageUrl', 'largeImageUrl'],
                                              [self.small_image_url, self.large_image_url]) if v is not None}
                card['image'] = image
            return card
        return {
            'type': 'Simple',
            'title': self.title,
            'content': self.content
        }


class Response(object):

    def __init__(self, output=None, reprompt_text=None, should_end_session=None):
        self.output = output
        self.reprompt_text = reprompt_text
        self.should_end_session = should_end_session
        self.card = None
        self.directives = []

    def add_card(self, title, content=None):
        if content is None:
            content = self.output
        self.card = Card(title=title, content=content)
        return self

    def with_card(self, card):
        self.card = card
        return self

    def add_reprompt(self, reprompt_text=None):
        if reprompt_text is None:
            reprompt_text = self.output
        self.reprompt_text = reprompt_text
        return self

    def add_directive(self, directive):
        self.directives.append(directive)
        return self

    def prepare(self, session_attributes=None):
        response = {}
        if self.output is not None:
            if isinstance(self.output, SSML):
                response['outputSpeech'] = {
                    'type': 'SSML',
                    'ssml': self.output.content
                }
            else:
                response['outputSpeech'] = {
                    'type': 'PlainText',
                    'text': self.output
                }
        if self.reprompt_text is not None:
            if isinstance(self.reprompt_text, SSML):
                response['reprompt'] = {
                    'outputSpeech': {
                        'type': 'SSML',
                        'ssml': self.reprompt_text.content
                    }
                }
            else:
                response['reprompt'] = {
                    'outputSpeech': {
                        'type': 'PlainText',
                        'text': self.reprompt_text
                    }
                }
        if self.should_end_session is not None:
            response['shouldEndSession'] = self.should_end_session

        if self.directives:
            response['directives'] = [d.prepare() for d in self.directives]

        if self.card is not None:
            response['card'] = self.card.prepare()

        container = {
            'version': '1.0',
            'response': response
        }
        if session_attributes:
            container['sessionAttributes'] = session_attributes

        return container

    @staticmethod
    def respond(output):
        return Response(output=output, should_end_session=False)

    @staticmethod
    def finish(output):
        return Response(output=output, should_end_session=True)

    @staticmethod
    def direct(directive, output=None):
        return Response(output=output, should_end_session=True).add_directive(directive=directive)


class AudioPlayer(object):

    def __init__(self, request_json):
        try:
            ap = request_json['context']['AudioPlayer']
            self.__token = ap.get('token', None)
            self.__offset_in_milliseconds = ap.get('offsetInMilliseconds', None)
            self.__player_activity = PLAYER_ACTIVITY(ap.get('playerActivity', None))
        except KeyError:
            raise ValueError('No AudioPlayer information in Request.')

    @property
    def token(self):
        return self.__token

    @property
    def offset_in_milliseconds(self):
        return self.__offset_in_milliseconds

    @property
    def player_activity(self):
        return self.__player_activity


class Request(object):

    def __init__(self, request_json):
        self.j = request_json
        try:
            self.__audio_player = AudioPlayer(request_json)
        except ValueError:
            self.__audio_player = None

    @property
    def new_session(self):
        try:
            return self.j['session']['new']
        except KeyError:
            pass
        return False

    @property
    def session_attributes(self):
        try:
            return self.j['session']['attributes']
        except KeyError:
            self.j['session']['attributes'] = {}
        return self.j['session']['attributes']

    @property
    def audio_player(self):
        return self.__audio_player

    @property
    def request_type(self):
        return self.j['request']['type']

    @staticmethod
    def wrap(request_json):
        try:
            request_type = request_json['request']['type']
            for sc in Request.__subclasses__():
                if sc.__name__ == request_type:
                    return sc(request_json)
        except KeyError:
            pass
        return Request(request_json=request_json)


class IntentRequest(Request):

    def __init__(self, request_json):
        super(IntentRequest, self).__init__(request_json=request_json)

    @property
    def intent_name(self):
        return self.j['request']['intent']['name']

    def get_slots(self):
        try:
            raw_slots = self.j['request']['intent']['slots']
            return {k: v['value'] for k, v in raw_slots.items() if 'value' in v}
        except KeyError:
            pass
        return {}


class Skill(object):

    def __init__(self):
        self._app_id = None

    def on_session_start(self, request):
        logger.info('Starting new session.')

    def on_launch_request(self, request):
        logger.info('Received Launch Request.')
        return Response.respond("Welcome")

    def on_session_ended_request(self, request):
        logger.info('Received Session Ended Request.')

    def on_default_intent_request(self, request):
        logger.info('Received un-handled IntentRequest: {}'.format(request.intent_name))
        return Response.respond("I'm afraid that I didn't understand that request, please try again.").add_reprompt()

    def dispatch(self, request):

        response = None
        if request.request_type == "SessionEndedRequest":
            self.on_session_ended_request(request=request)
        elif request.request_type == "LaunchRequest":
            response = self.on_launch_request(request=request)
        elif isinstance(request, IntentRequest):
            f = getattr(self, self.mangle(request.intent_name), self.on_default_intent_request)
            response = f(request)
        return response

    def handler(self, event, context):

        request = Request.wrap(event)

        if self._app_id is not None:
            if request.j['session']['application']['applicationId'] != self._app_id:
                raise ValueError("Invalid Application ID")

        if request.new_session:
            self.on_session_start(request)

        response = self.dispatch(request=request)

        if response:
            return response.prepare(session_attributes=request.session_attributes)
        return

    @staticmethod
    def mangle(intent_name):
        return "do_" + intent_name.lower().replace("amazon.", "AMZ_") \
            .replace("<", "_LT_") \
            .replace(">", "_GT") \
            .replace("@", "_AT_") \
            .replace("[", "_LB_") \
            .replace("]", "_RB_")

    @classmethod
    def intent(cls, intent_name):

        mangled = cls.mangle(intent_name)

        def decorator(f):
            setattr(cls, mangled, f)
            return f

        return decorator

    @classmethod
    def get_handler(cls):
        return cls().handler



import json
import re


class Expandable(object):

    SEARCH_PAT = re.compile("(<[^<>]+>)")

    def __init__(self, text):
        self.text = text

    def expand(self):
        utterances = []
        queue = [self.text]
        while len(queue) > 0:
            utterance = queue.pop()
            match = self.SEARCH_PAT.search(utterance)
            if match is None:
                utterances.append(utterance)
            else:
                alt_group = match.groups()[0]
                alternates = alt_group.strip('<>').split('|')
                for alternate in alternates:
                    queue.append(utterance.replace(alt_group, alternate, 1))
        return {u.strip() for u in utterances}


class LanguageModelIntentSlot(object):

    def __init__(self, name, slot_type, samples=None):
        self.name = name
        self.type = slot_type
        self.samples = samples if samples else []

    def to_json(self):
        o = {"name": self.name,
             "type": self.type}
        if self.samples:
            samples = set()
            for sample in self.samples:
                if isinstance(sample, Expandable):
                    samples = samples.union(sample.expand())
                else:
                    samples.add(sample)
            o["samples"] = list(samples)
        return o


class LanguageModelIntent(object):

    def __init__(self, name):
        self.name = name
        self.samples = []
        self.slots = {}

    def add_slot(self, name, slot_type):
        slot = LanguageModelIntentSlot(name=name, slot_type=slot_type)
        self.slots[name] = slot
        return slot

    def to_json(self):
        o = {"name": self.name}
        if self.samples:
            samples = set()
            for sample in self.samples:
                if isinstance(sample, Expandable):
                    samples = samples.union(sample.expand())
                else:
                    samples.add(sample)
            o["samples"] = list(samples)
        if self.slots:
            o["slots"] = [slot.to_json() for slot in self.slots.values()]
        return o


class LanguageModelSlotTypeValue(object):

    def __init__(self, value, id=None, synonyms=None):
        self.value = value
        self.id = id
        self.synonyms = synonyms

    def to_json(self):
        o = {"id": self.id, "name": {"value": self.value}}
        if self.synonyms:
            o["name"]["synonyms"] = self.synonyms
        return o


class LanguageModelSlotType(object):

    def __init__(self, name):
        self.name = name
        self.values = []

    def add_value(self, value):
        v = LanguageModelSlotTypeValue(value=value)
        self.values.append(v)
        return v

    def add_values(self, values):
        self.values.extend([LanguageModelSlotTypeValue(v) for v in values])

    def to_json(self):
        return {"name": self.name, "values": [v.to_json() for v in self.values]}


class LanguageModel(object):

    def __init__(self, invocation_name):
        self.invocationName = invocation_name
        self.types = {}
        self.intents = {}

    def add_intent(self, name):
        intent = LanguageModelIntent(name=name)
        self.intents[name] = intent
        return intent

    def add_slot_type(self, name):
        slot_type = LanguageModelSlotType(name=name)
        self.types[name] = slot_type
        return slot_type

    def to_json(self):
        o = {"invocationName": self.invocationName}
        if self.intents:
            o["intents"] = [i.to_json() for i in self.intents.values()]
        if self.types:
            o["types"] = [t.to_json() for t in self.types.values()]
        return o


class PromptVariation(object):

    PLAIN = "PlainText"
    SSML = "SSML"
    AUTO = None

    def __init__(self, value, text_type=None):
        self.value = value
        if text_type == None:
            if '<' in value:
                self.type = PromptVariation.SSML
            else:
                self.type = PromptVariation.PLAIN
        else:
            self.type = text_type

    def to_json(self):
        return {"type": self.type,
                "value": self.value}


class Prompt(object):

    def __init__(self, id_string):
        self.id = id_string
        self.variations = []

    def add_variation(self, value, text_type=PromptVariation.AUTO):
        self.variations.append(PromptVariation(value=value, text_type=text_type))

    def to_json(self):
        if not self.variations:
            raise ValueError("At least one utterance must be provided for each Prompt.")
        return {"id": self.id,
                "variations": [v.to_json() for v in self.variations]}


class DialogIntentSlot(object):

    ELICIT = "elicit"
    CONFIRMATION = "confirmation"

    def __init__(self, name, slot_type, elicitation_required=True, confirmation_required=False):
        self.name = name
        self.type = slot_type
        self.elicitationRequired = elicitation_required
        self.confirmationRequired = confirmation_required
        self.prompts = {}

    def add_prompt(self, prompt_id, prompt_type="elicit"):
        self.prompts[prompt_type] = prompt_id

    def to_json(self):
        if self.confirmationRequired and "confirmation" not in self.prompts:
            raise ValueError("Dialog Slot Requires Confirmation but has No Confirmation Prompt.")
        if self.elicitationRequired and "elicit" not in self.prompts:
            raise ValueError("Dialog Slot Requires Elicitation but has not Elicitation Prompt.")
        return {"name": self.name,
                "type": self.type,
                "elicitationRequired": self.elicitationRequired,
                "confirmationRequired": self.confirmationRequired,
                "prompts": self.prompts}


class DialogIntent(object):

    def __init__(self, name, confirmation_required=False):
        self.name = name
        self.confirmationRequired = confirmation_required
        self.prompts = {}
        self.slots = {}

    def add_slot(self, name, slot_type, elicitation_required=True, confirmation_required=False):
        slot = DialogIntentSlot(name=name,
                                slot_type=slot_type,
                                elicitation_required=elicitation_required,
                                confirmation_required=confirmation_required)
        self.slots[name] = slot
        return slot

    def add_prompt(self, prompt_id):
        self.prompts["confirmation"] = prompt_id

    def to_json(self):
        if self.confirmationRequired and "confirmation" not in self.prompts:
            raise ValueError("Dialog Intent Requires Confirmation but has No Confirmation Prompt.")
        return {"name": self.name,
                "confirmationRequired": self.confirmationRequired,
                "prompts": self.prompts,
                "slots": [s.to_json() for s in self.slots.values()]}


class Dialog(object):

    def __init__(self):
        self.intents = {}

    def add_intent(self, name, confirmation_required=False):
        i = DialogIntent(name=name, confirmation_required=confirmation_required)
        self.intents[name] = i
        return i

    def to_json(self):
        return [i.to_json() for i in self.intents.values()]


class InteractionModel(object):

    def __init__(self):
        self.languageModel = None
        self.dialog = None
        self.prompts = {}

    def add_language_model(self, invocation_name):
        self.languageModel = LanguageModel(invocation_name=invocation_name)
        return self.languageModel

    def add_prompt(self, id_string):
        p = Prompt(id_string=id_string)
        self.prompts[id_string] = p
        return p

    def add_dialog_intent(self, name, confirmation_required=False):
        if not self.dialog:
            self.dialog = Dialog()
        return self.dialog.add_intent(name=name, confirmation_required=confirmation_required)

    def to_json(self):
        o = {}
        if self.languageModel:
            o["languageModel"] = self.languageModel.to_json()
        if self.prompts:
            o["prompts"] = [p.to_json() for p in self.prompts.values()]
        if self.dialog:
            o["dialog"] = self.dialog.to_json()
        return {"interactionModel": o}

    def save(self, filename):
        with open(filename, 'wt') as modelfile:
            json.dump(self.to_json(), modelfile, indent=2)



import enum


class SIZE(enum.Enum):
    X_SMALL = "X_SMALL"
    SMALL = "SMALL"
    MEDIUM = "MEDIUM"
    LARGE = "LARGE"
    X_LARGE = "X_LARGE"


class ImageSource(object):

    def __init__(self, url, size=None, width_pixels=None, height_pixels=None):
        if not url:
            raise ValueError('"url" must not be None.')
        self.url = url
        self.size = size
        if self.size:
            self.size = SIZE(self.size)
        self.width_pixels = width_pixels
        self.heigh_pixels = height_pixels

    def prepare(self):
        source = {'url': self.url}
        if self.size:
            source['size'] = self.size.value
        if self.width_pixels and self.heigh_pixels:
            source['widthPixels'] = self.width_pixels
            source['heightPixels'] = self.heigh_pixels
        return source

class Image(object):

    def __init__(self, content_description, sources=None):
        self.content_description = content_description
        self.sources = []
        if sources:
            self.sources.extend(sources)

    def add_source(self, url, size=None, width_pixels=None, height_pixels=None):
        self.sources.append(ImageSource(url=url, size=size, width_pixels=width_pixels, height_pixels=height_pixels))
        return self

    def with_source(self, source):
        if source:
            self.sources.append(source)
        return self

    def prepare(self):
        if not self.sources:
            raise RuntimeError('No sources for image.')
        return {
            'contentDescription': self.content_description,
            'sources': [s.prepare() for s in self.sources]
        }



import boto3


class PersistenceException(Exception):
    pass


class AttributeStore(object):

    def __init__(self, table_name, region_name=None, endpoint_url=None,
                 create_table=False, read_capacity_units=5, write_capacity_units=5):
        self.__dbservice = boto3.resource('dynamodb', region_name=region_name, endpoint_url=endpoint_url)
        try:
            self.__table = self.__dbservice.Table(table_name)
        except self.__dbservice.exceptions.ResourceNotFoundException as e:
            if not create_table:
                raise PersistenceException('Persistence table not found.')
            self.__table = self.__dbservice.create_table(
                TableName=table_name,
                KeySchema=[
                    {
                        'AttributeName': 'userId',
                        'KeyType': 'HASH'
                    },
                    {
                        'AttributeName': 'deviceId',
                        'KeyType': 'RANGE'
                    }
                ],
                AttributeDefinitions=[
                    {
                        'AttributeName': 'userId',
                        'AttributeType': 'S'
                    },
                    {
                        'AttributeName': 'deviceId',
                        'AttributeType': 'S'
                    }
                ],
                ProvisionedThroughput={
                    'WriteCapacityUnits': write_capacity_units,
                    'ReadCapacityUnits': read_capacity_units
                }
            )

    def load(self, request):
        context = request.j['context']['System']
        userid = context['user']['userId']
        deviceid = context['device']['deviceId']
        try:
            entry = self.__table.get_item(Key={'userId': userid, 'deviceId': deviceid})['Item']
        except KeyError:
            _ = self.__table.put_item(Item={'userId': userid, 'deviceId': deviceid, 'Attributes': {}})
            entry = self.__table.get_item(Key={'userId': userid, 'deviceId': deviceid})['Item']
        request.session_attributes.update(entry['Attributes'])

    def save(self, request):
        context = request.j['context']['System']
        userid = context['user']['userId']
        deviceid = context['device']['deviceId']
        _ = self.__table.update_item(Key={'userId': userid, 'deviceId': deviceid},
                                     UpdateExpression='set Attributes = :a',
                                     ExpressionAttributeValues={':a': request.session_attributes})



from setuptools import setup
import lambdaskill

def readme():
    with open('README.rst', 'rt') as readme_file:
        return readme_file.read()


setup(name='lambdaskill',
      version=lambdaskill.__version__,
      description='A simple toolkit for building Alexa skills.',
      long_description=readme(),
      author='Michael Uhl',
      url='https://github.com/michaeluhl/lambdaskill',
      license='LGPL',
      packages=['lambdaskill'],
      install_requires=[
          'aniso8601',
          ],
      classifiers=[
          'Development Status :: 4 - Beta',
          'Environment :: Other Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3 :: Only',
          'Topic :: Software Development :: Libraries :: Application Frameworks',
          ],
      zip_safe=True)
